import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import Treemap from './treemap';
import * as d3 from 'd3';

const dataset = {
  children: [
    {
      name: 'boss1',
      children: [
        { name: 'a', group: 'A', value: 28, colname: 'level1' },
        { name: 'b', group: 'A', value: 19, colname: 'level1' },
        { name: 'c', group: 'C', value: 18, colname: 'level1' },
        { name: 'd', group: 'C', value: 19, colname: 'level1' },
      ],
      colname: 'level1',
    },
    {
      name: 'boss2',
      children: [
        { name: 'e', group: 'C', value: 14, colname: 'level2' },
        { name: 'f', group: 'A', value: 11, colname: 'level2' },
        { name: 'g', group: 'B', value: 15, colname: 'level2' },
        { name: 'h', group: 'B', value: 16, colname: 'level2' },
      ],
      colname: 'level2',
    },
    {
      name: 'boss3',
      children: [
        { name: 'i', group: 'B', value: 10, colname: 'level3' },
        { name: 'j', group: 'A', value: 13, colname: 'level3' },
        { name: 'k', group: 'A', value: 13, colname: 'level3' },
        { name: 'l', group: 'D', value: 25, colname: 'level3' },
        { name: 'm', group: 'D', value: 16, colname: 'level3' },
        { name: 'n', group: 'D', value: 28, colname: 'level3' },
      ],
      colname: 'level3',
    },
  ],
  name: 'CEO',
};

const App = () => {
  const [data, setData] = useState(null);
  const [boss, setBoss] = useState(null);
  const [employee, setEmployee] = useState(null);

  useEffect(() => {
    init();
  }, []);

  const init = () => {
    setData(dataset);
    setBoss(dataset['children'].length);

    let cnt = 0;
    for (let i = 0; i < dataset['children'].length; i++)
      cnt += dataset['children'][i]['children'].length;

    setEmployee(cnt);
  };

  const deepCopy = (obj) => {
    return JSON.parse(JSON.stringify(obj));
  };

  const updateChildValue = () => {
    let _data = deepCopy(data);

    _data['children'][0]['children'][0]['value'] = 50;
    _data['children'][0]['children'][1]['value'] = 10;
    _data['children'][0]['children'][2]['value'] = 30;

    _data['children'][1]['children'][0]['value'] = 4;
    _data['children'][1]['children'][1]['value'] = 8;

    setData(_data);
  };

  const addAndRemoveEmployee = () => {
    let _data = deepCopy(data);

    _data['children'][0]['children'].push({
      name: 'p',
      group: 'C',
      value: 20,
      colname: 'level3',
    });

    _data['children'][2]['children'].splice(0, 1);

    setData(_data);
  };

  const addBoss = () => {
    let _data = deepCopy(data);

    _data['children'].push({
      name: 'boss' + (boss + 1),
      children: [
        {
          name: 'z',
          group: 'E',
          value: 40,
          colname: 'level3',
        },
      ],
      colName: 'level3',
    });

    setData(_data);
    setBoss(boss + 1);
    setEmployee(employee + 1);
  };

  const removeBoss = () => {
    let _data = deepCopy(data);

    let removed = _data['children'].splice(0, 1);
    setBoss(boss - 1);
    setEmployee(employee - removed[0]['children'].length);

    setData(_data);
    console.log(_data);
  };

  const resetData = () => {
    init();
  };

  if (data === null) {
  	return <></>
  }
    
  return (
    <div className="App">
      <h2>
        {boss} group leaders and {employee} employees
      </h2>
      <div className="btns">
        <button onClick={updateChildValue}>Change Child Data Values</button>
        <button onClick={addAndRemoveEmployee}>Add/Remove Child Nodes</button>
        <button onClick={addBoss}>Add Parent Nodes</button>
        <button onClick={removeBoss}>Remove Parent Nodes</button>
        <button onClick={resetData}>Reset</button>
      </div>
      <Treemap width={800} height={400} data={data} />
    </div>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
