import * as d3 from 'd3';
import React, { useRef, useEffect } from 'react';

const treeMap = ({ width, height, data }) => {
  const ref = useRef();
  
  useEffect(() => {
  	const svg = d3.select(ref.current)
			.attr("width", width)
    	.attr("height", height)
    	.style("border", "1px solid black")
  }, []);
  
  useEffect(() => {
  	draw();
  }, [data]);
  
  const draw = () => {
    if (data.children.length === 0)
      return;
    
    const svg = d3.select(ref.current);
    
    let root = d3.hierarchy(data)
    	.sum(d => d.value);
    
    d3.treemap()
    	.size([width, height])
    	.paddingTop(28)
    	.paddingRight(7)
    	.paddingInner(3)
    	(root);
    
    const color = d3.scaleOrdinal()
    	.domain(["boss1", "boss2", "boss3"])
    	.range(["#402D54", "#D18975", "#8FD175"]);
    
    const opacity = d3.scaleLinear()
    	.domain([1, 30])
    	.range([1, 0.5]);
    
    let nodes = svg.selectAll("rect")
    	.data(root.leaves());
    
    nodes.transition().duration(3000)
    	.attr("x", d => d.x0)
    	.attr("y", d => d.y0)
    	.attr("width", d => d.x1 - d.x0)
    	.attr("height", d => d.y1 - d.y0)
    	.style("opacity", d => opacity(d.data.value))
    	.style("fill", d => color(d.parent.data.name))
    
    nodes.enter()
    	.append("rect")
    	.attr("x", d => d.x0)
    	.attr("y", d => d.y0)
    	.attr("width", d => d.x1 - d.x0)
    	.attr("height", d => d.y1 - d.y0)
    	.style("stroke", "black")
    	.style("fill", d => color(d.parent.data.name))
    	.style("opacity", d => opacity(d.data.value))
    
    nodes.exit().remove()
    
    let nodeText = svg.selectAll("text")
    	.data(root.leaves())
    
    nodeText.transition().duration(3000)
    	.attr("x", d => d.x0 + 5)
    	.attr("y", d => d.y0 + 20)
    	.text(d => d.data.name)
    
    nodeText.enter()
    	.append("text")
    	.attr("x", d => d.x0 + 5)
    	.attr("y", d => d.y0 + 20)
    	.text(d => d.data.name)
    	.attr("font-size", "19px")
    	.attr("fill", "white")
    
    nodeText.exit().remove()
    
    let nodeVals = svg.selectAll("vals")
    	.data(root.leaves())
    
    nodeVals.transition().duration(3000)
    	.attr("x", d => d.x0 + 5)
    	.attr("y", d => d.y0 + 35)
    	.text(d => d.data.value)
    
    nodeVals.enter()
    	.append("text")
    	.attr("x", d => d.x0 + 5)
    	.attr("y", d => d.y0 + 35)
    	.text(d => d.data.value)
    	.attr("font-size", "11px")
    	.attr("fill", "white")
    
    nodeVals.exit().remove();
    
    svg.selectAll("titles")
    	.data(root.descendants().filter(d => d.depth == 1))
    	.enter()
    	.append("text")
    	.attr("x", d => d.x0)
    	.attr("y", d => d.y0 + 21)
    	.text(d => d.data.name)
    	.attr("font-size", "19px")
    	.attr("fill", d => color(d.data.name))    
  }
  
  return (
    <div className="chart">
    	<svg ref={ref}>
      </svg>
    </div>
  )
}

export default treeMap;